from rest_framework import serializers
from django.db.models import Max
from brsessionApp.models import PatientSession,PatientSessionItem,Session,PtSsnPayment,SessionStatus
from brpatientApp.models import PatientInfo

class PatientSessionSerializer(serializers.ModelSerializer):
    session_string = serializers.SerializerMethodField()
    patient_string = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(format="%Y-%m-%d",required=False)
    class Meta:
        model = PatientSession
        fields="__all__"
        #extra_kwargs = {'created_at':{'required': False}}
        read_only_fields = ('session_string', 'patient_string')

    def get_session_string(self, obj):
        return obj.session.name    
    def get_patient_string(self, obj):
        return obj.patient.fullname

class PatientSessionItemCustomSerializer(serializers.ModelSerializer):
    dr_name = serializers.CharField(source='doctor.name',read_only=True)
    patient_id = serializers.CharField(source='session.patient_id',read_only=True)
    pt_fullname = serializers.CharField(source='session.patient.fullname',read_only=True)
    pt_number_1 = serializers.CharField(source='session.patient.number_1',read_only=True)
    pt_number_2 = serializers.CharField(source='session.patient.number_2',read_only=True)
    hs_name = serializers.CharField(source='hour_slot.name',read_only=True)
    s_amount_remaining = serializers.IntegerField(source='session.amount_remaining',read_only=True)
    s_amount_required = serializers.IntegerField(source='session.amount_required',read_only=True)
    s_count = serializers.IntegerField(source='session.sessions_count',read_only=True)
    s_name = serializers.CharField(source='session.session.name',read_only=True)
    entry_status_str = serializers.CharField(source='get_entry_status_display',read_only=True)
    session_status_str = serializers.CharField(source='session_status.name',read_only=True)
    session_status_color = serializers.CharField(source='session_status.color_code',read_only=True)
    s_is_package = serializers.BooleanField(source='session.is_package',read_only=True)
    #very special relation to move around models 
    #source='session.patient.PatientInfo_patient.pt_status.name', 
    pt_status_name = serializers.CharField(
        source='session.patient.patient_info.pt_status.name',  # should set patient fk  related_name="patient_info", to point to patient object relation 
        read_only=True,default="",
    )
    #entry_datetime
    #get_status_display
    class Meta:
        model = PatientSessionItem
        fields="__all__"
    def to_representation(self, instance):
        obj = super().to_representation(instance)
        #obj["doctor"]= instance.doctor.get_full_name() if instance.doctor else ""
        obj['session_day'] = instance.session_day.strftime("%Y-%m-%d")
        obj['entry_datetime'] = instance.entry_datetime.strftime("%I:%M %p") if instance.entry_datetime else ""
        obj['finish_datetime'] = instance.finish_datetime.strftime("%I:%M %p") if instance.finish_datetime else ""
        #obj['status'] = instance.get_status_display()
        return obj
    
class PatientSessionItemSerializer(serializers.ModelSerializer):
    class Meta:
        model = PatientSessionItem
        fields="__all__"
    def to_representation(self, instance):
        #return PatientSessionItemCustomSerializer(instance).data  # Create an instance of the custom serializer and access its data
        return PatientSessionItemCustomSerializer(instance).data
        #return super(PatientSessionItemCustomSerializer, self).to_representation(instance)   
    
    #     def to_representation(self, instance):
    #     return super(UserSerializer, self).to_representation(instance) 

class PtSsnPaymentSerializer(serializers.ModelSerializer):
    created_by_name = serializers.CharField(source="created_by.name",default="",read_only=True)
    created_at = serializers.DateTimeField(format="%I:%M %p @ %Y-%m-%d",read_only=True)
    class Meta:
        model = PtSsnPayment
        fields="__all__"